﻿using System;
using System.Data.Common;
using System.Configuration;
using System.Data;
using CYQ.Data.SQL;
using System.Diagnostics;

namespace CYQ.Data.DAL
{

    /// <summary>
    /// 数据库操作基类
    /// </summary>
    internal abstract class DbBase : IDisposable
    {
        public bool WriteLog = true;
        public DalType dalType = DalType.Sql;
        internal System.Text.StringBuilder attachInfo = new System.Text.StringBuilder();
        internal string tempSql = string.Empty;//附加信息，包括调试信息
        internal bool openTrans = false;
        internal IsolationLevel tranLevel = IsolationLevel.Unspecified;
        private DbProviderFactory _fac = null;
        private DbConnection _con = null;
        private DbCommand _com;
        private DbTransaction _tran;
        private Stopwatch watch;
        public DbConnection Con
        {
            get
            {
                return _con;
            }
        }
        public DbCommand Com
        {
            get
            {
                return _com;
            }
        }
        /// <summary>
        /// 获得链接的数据库名称
        /// </summary>
        public string DataBase
        {
            get
            {
                if (!string.IsNullOrEmpty(_con.Database))
                {
                    return _con.Database;
                }
                return System.IO.Path.GetFileNameWithoutExtension(_con.DataSource);
            }
        }
        /// <summary>
        /// 是否允许记录SQL语句
        /// </summary>
        private bool AllowRecordSql
        {
            get
            {
                return AppConfig.OpenDebugInfo || AppConfig.RecordSqlLongQueryTime > 0;
            }
        }
        public DbBase(string conn, string providerName)
        {
            dalType = DalAction.GetDalType(providerName);
            _fac = GetFactory(providerName);
            _con = _fac.CreateConnection();
            _con.ConnectionString = FormatConn(conn);
            _com = _con.CreateCommand();
            _com.Connection = _con;
            if (AllowRecordSql)//开启秒表计算
            {
                watch = new Stopwatch();
            }
        }
        protected virtual DbProviderFactory GetFactory(string providerName)
        {
            return DbProviderFactories.GetFactory(providerName);
        }
        private string FormatConn(string connString)
        {
            if (dalType != DalType.Access)
            {
                string conn = connString.ToLower();
                int index = conn.IndexOf("provider");
                if (index > -1)
                {
                    int end = conn.IndexOf(';', index);
                    if (end > index)
                    {
                        connString = conn.Remove(index, end - index + 1);
                    }
                }
            }
            return connString;
        }
        private int returnValue = -1;
        public int ReturnValue
        {
            get
            {
                if (returnValue == -1 && _com != null && _com.Parameters != null)
                {
                    int.TryParse(Convert.ToString(_com.Parameters[_com.Parameters.Count - 1].Value), out returnValue);
                }
                return returnValue;
            }
            set
            {
                returnValue = value;
            }
        }
        public virtual string Pre
        {
            get
            {
                return "@";
            }
        }
        #region 执行
        public DbDataReader ExeDataReader(string procName, bool isProc)
        {
            SetCommandText(procName, isProc);
            DbDataReader sdr = null;
            try
            {
                OpenCon();
                sdr = _com.ExecuteReader(openTrans ? CommandBehavior.Default : CommandBehavior.CloseConnection);
                if (sdr != null && !sdr.HasRows)
                {
                    sdr.Close();
                    sdr = null;
                }

            }
            catch (DbException err)
            {
                WriteError(err.Message);
            }
            finally
            {
                WriteTime();
            }
            return sdr;
        }
        public int ExeNonQuery(string procName, bool isProc)
        {
            SetCommandText(procName, isProc);
            int rowCount = 1;
            try
            {
                OpenCon();
                _com.ExecuteNonQuery();
            }
            catch (DbException err)
            {
                rowCount = 0;
                WriteError(err.Message);
            }
            finally
            {
                if (!openTrans)
                {
                    CloseCon();
                }
                WriteTime();
            }
            return rowCount;
        }
        public object ExeScalar(string procName, bool isProc)
        {
            SetCommandText(procName, isProc);
            object returnValue = null;
            try
            {
                OpenCon();
                returnValue = _com.ExecuteScalar();
            }
            catch (DbException err)
            {
                WriteError(err.Message);
            }
            finally
            {
                if (!openTrans)
                {
                    CloseCon();
                }
                WriteTime();
            }
            return returnValue;
        }
        public DataTable ExeDataTable(string procName, bool isProc)
        {
            SetCommandText(procName, isProc);
            DbDataAdapter sdr = _fac.CreateDataAdapter();
            sdr.SelectCommand = _com;
            DataTable dataTable = new DataTable();
            try
            {
                OpenCon();
                sdr.Fill(dataTable);
            }
            catch (DbException err)
            {
                WriteError(err.Message);
            }
            finally
            {
                sdr.Dispose();
                if (!openTrans)
                {
                    CloseCon();
                }
                WriteTime();
            }
            return dataTable;
        }
        #endregion
        public void AddParameters(string parameterName, object value)
        {
            AddParameters(parameterName, value, DbType.String, -1, ParameterDirection.Input);
        }
        public virtual void AddParameters(string parameterName, object value, DbType dbType, int size, ParameterDirection direction)
        {
            if (dalType == DalType.Oracle)
            {
                parameterName = parameterName.Replace(":", "").Replace("@", "");
            }
            else
            {
                parameterName = parameterName.Substring(0, 1) == Pre ? parameterName : Pre + parameterName;
            }
            DbParameter para = GetNewParameter();
            para.ParameterName = parameterName;
            para.Value = value;
            para.DbType = dbType;

            if (dbType != DbType.Binary && size > -1)
            {
                para.Size = size;
            }
            para.Direction = direction;

            Com.Parameters.Add(para);
        }
        internal virtual void AddCustomePara(string paraName, ParaType paraType) { }
        public abstract DbParameter GetNewParameter();

        public void ClearParameters()
        {
            if (_com != null && _com.Parameters != null)
            {
                _com.Parameters.Clear();
            }
        }
        public abstract void AddReturnPara();

        private void SetCommandText(string commandText, bool isProc)
        {
            _com.CommandText = isProc ? commandText : SQLString.FormatDal(commandText, dalType, false);
            _com.CommandType = isProc ? CommandType.StoredProcedure : CommandType.Text;
            if (isProc && commandText.Contains("SelectBase") && !_com.Parameters.Contains("ReturnValue"))
            {
                AddReturnPara();
            }
            else
            {
                string checkText = commandText.ToLower();
                if (checkText.IndexOf("table") > -1 && (checkText.IndexOf("delete") > -1 || checkText.IndexOf("drop") > -1 || checkText.IndexOf("truncate") > -1))
                {
                    Log.WriteLog(commandText);
                }
            }
            if (AllowRecordSql)
            {
                tempSql = GetParaInfo(commandText) + "<br>execute time is: ";
            }
        }
        private string GetParaInfo(string commandText)
        {
            string paraInfo = "<hr>" + dalType + "." + DataBase + ".SQL:<br> " + commandText;
            foreach (DbParameter item in _com.Parameters)
            {
                paraInfo += "<br>Para: " + item.ParameterName + "->" + item.Value;
            }
            return paraInfo;
        }
        /// <summary>
        /// 记录执行时间
        /// </summary>
        private void WriteTime()
        {
            if (AllowRecordSql)
            {
                watch.Stop();
                tempSql += watch.Elapsed.TotalMilliseconds + " (ms)";
                if (AppConfig.OpenDebugInfo)
                {
                    attachInfo.Append(tempSql);
                    if (AppDebug.IsRecording && watch.Elapsed.TotalMilliseconds >= AppConfig.AppDebugFilterTime)
                    {
                        AppDebug.Add(tempSql);
                    }
                }
                if (watch.Elapsed.TotalMilliseconds >= AppConfig.RecordSqlLongQueryTime)
                {
                    Log.WriteLogToTxt(tempSql);
                }
                watch.Reset();
                tempSql = null;
            }
        }
        #region IDisposable 成员

        public void Dispose()
        {

            if (_con != null)
            {
                CloseCon();
                _con = null;
            }
            if (_com != null)
            {
                _com = null;
            }
        }
        private void OpenCon()
        {
            try
            {
                if (_con.State == ConnectionState.Closed)
                {
                    _con.Open();
                }
                if (openTrans && _tran == null)
                {
                    _tran = _con.BeginTransaction(tranLevel);
                    _com.Transaction = _tran;
                }
                if (AllowRecordSql)
                {
                    watch.Start();
                }
            }
            catch (DbException err)
            {
                WriteError(err.Message);
            }

        }
        private void CloseCon()
        {
            try
            {
                if (_con.State == ConnectionState.Open)
                {
                    if (_tran != null)
                    {
                        openTrans = false;
                        _tran.Commit();
                        _tran = null;
                    }
                    _con.Close();
                }
            }
            catch (DbException err)
            {
                WriteError(err.Message);
            }

        }
        public void EndTransaction()
        {
            openTrans = false;
            if (_tran != null)
            {
                try
                {
                    _tran.Commit();
                }
                catch (Exception err)
                {
                    WriteError(err.Message);
                }
                finally
                {
                    _tran = null;
                }
            }
        }
        public void RollBack()
        {
            if (_tran != null)
            {
                try
                {
                    _tran.Rollback();
                }
                catch (Exception err)
                {
                    WriteError(err.Message);
                }
                finally
                {
                    _tran = null;
                }
            }
        }
        internal delegate void OnException(string msg);
        internal event OnException OnExceptionEvent;
        private void WriteError(string err)
        {
            if (OnExceptionEvent != null)
            {
                OnExceptionEvent(err);
            }
            if (_tran != null)
            {
                _tran.Rollback();
                _tran = null;
            }
            if (WriteLog)
            {
                Log.WriteLog(err + attachInfo);
            }

        }
        #endregion
    }
    /// <summary>
    /// 特殊参数类型[MProc操作Oracle时用到]
    /// </summary>
    public enum ParaType
    {
        /// <summary>
        /// 游标类型
        /// </summary>
        Cursor,
        /// <summary>
        /// 输出类型
        /// </summary>
        OutPut,
        /// <summary>
        /// 返回值类型
        /// </summary>
        ReturnValue,
    }
}